"use strict";
{
  let readFile;
  const fs = require('fs');
  if ( !! fs.readFile ) {
    const {promisify} = require('util');
    readFile = async (...args) => await promisify(fs.readFile)(...args);
  } else {
    readFile = async (...args) => {
      const resp = await fetch(...args);
      return await resp.text();
    };
  }
  const { T, FutureType, execute, typeOf } = require('./futuretype.js');
  const I = {};

  let parser;

  module.exports = { def, T, I, renderHTML };

  function def( template, ...values ) {
    let contents;
    template = Array.from( template );
    values = Array.from( values );
    
    // the first value in template is the name
    
    let component_name = template.shift().trim();
    
    // the first value in values is a spec object 

    const sig = values.shift() || {};
    
    if ( component_name.length == 0 ) {
      component_name = sig.name || '';
    }

    const comp = async function component( data ) {
      if ( !! sig.file ) {
        let stylesheet = '';
        let viewport = '';
        let script = '';
        if ( !! sig.script ) {
          script = `<script src="${sig.script}"></script>`;
        }
        if ( !! sig.stylesheet ) {
          stylesheet = `<link rel=stylesheet href="${sig.stylesheet}">`;
        }
        if ( ! sig.no_viewport ) {
          viewport = `<!DOCTYPE html>
            <meta charset=utf-8>
            <meta name=viewport content="width=device-width, initial-scale=1, user-scalable=no">`;
        }
        if ( contents == undefined ) {
          contents = await readFile( sig.file );
        }
        function set_contents( contents_template, ...contents_values ) {
          template = Array.from( contents_template );
          values = Array.from( contents_values );
        }
        eval("set_contents`" + viewport + stylesheet + script + contents + "`");
      }

      if ( typeOf( data ) == "FutureType" ) {
        comp.$$future = data;
        return comp;
      }
      if ( typeOf( data ) == "Promise" ) {
        data = await data;
      }

      const resolvedValues = await Promise.all( values.map( async val => await resolveVal( data, val ) ) );

      let markup = weave( template, resolvedValues ).join('');

      if ( sig.requests_iframe ) {
        markup = add_iframe( { markup, component_name, sig } ) ;
      }

      return markup;
    }

    if ( !!component_name ) {
      I[component_name] = comp;
      comp.cname = component_name;
    }


    return comp;
  }
                                        
  // simple things
async function resolveVal( data, val ) {
  if ( typeOf( val ) == "Promise" ) {
    val = await val;
  } else if ( typeOf( val ) == "FutureType" ) {
    val = await execute( data, val ); 
  }
  if ( !! val && typeOf( val.$$future ) == "FutureType" ) {
    val = await val( await execute( data, val.$$future ), true ); 
  } 
  if ( typeOf( val ) == "AsyncFunction" ) {
    val = await val( data );
  }
  if ( typeOf( val ) == "Function" ) {
    val = val( data );
  }
  if ( typeOf( val ) == "Array" ) {
    val = val.join(' \n');
  }
  return val;
}

    function safe( s ) {
      return s.replace(/&/g, '&amp;' ).replace(/>/g, '&gt;').replace(/</g,'&lt;');
    }

    function add_iframe( { markup, component_name, sig } ) {
      markup = markup.replace( /"/g, '&quot;' );
      markup = `<iframe
        data-component-name="${component_name}"
        seamless=seamless
        frameBorder=0
        srcdoc="${markup}"
        src="data:text/html,${markup}"
        ${ sig.iframe_name ? `name="${sig.iframe_name}"` : '' }
        width=${sig.width}
        height=${sig.height}
        >
      </iframe>`
      return markup;
    }

    function weave( long, short ) {
      try { 
        const lace = new Array( long.length + short.length );
        for( let i = 0, j = 0; i < lace.length; ) {
          lace[i] = long[j];
          i += 1;
          if ( j < short.length ) {
            lace[i] = short[j];
            i+= 1;
          }
          j+= 1;
        }
        return lace;
      } catch(e) {
        console.warn(e);
      }
    }

    function renderHTML( markup_string ) {
      if ( ! parser ) {
        parser = new DOMParser();
      }
      return parser.parseFromString(`<template>${markup_string}</template>`,"text/html").head.firstChild.content;
    }
}
      
