"use strict";
{
  module.exports = { dom1, domn, adomn };

  function dom1( s ) {
    return document.querySelector( s );
  }

  function domn( s ) {
    return Array.from( document.querySelectorAll( s ) );
  }

  // NOTE
    // this ensures that we get the elements referred to by selector s
    // it will keep checking until it gets them
    // at intervals of 15ms
  async function adomn( s, maxwait = 0, firstTime = maxwait > 0 ? + new Date : 0 ) {
    const el = domn(s);
    if ( el.length ) {
      return el;
    }
    const loadready = new Promise( r => {
      self.addEventListener( 'load', () => {
        setTimeout( () => r( adomn(s, maxwait, firstTime) ), 0 );
      });
    });
    const contentready = new Promise( r => {
      document.addEventListener( 'domcontentloaded', () => {
        setTimeout( () => r( adomn(s, maxwait, firstTime) ), 0 );
      });
    });
    const timer = new Promise( r => {
      if ( maxwait ) {
        const timeNow = + new Date;
        if ( ( timeNow - firstTime ) > maxwait ) {
          console.log( `Wait for ${s} timed out after ${maxwait} ms.` );
          return r( [] );
        }
      }
      setTimeout( () => r( adomn(s, maxwait, firstTime) ), 15 );
    });
    return Promise.race( [ loadready, contentready, timer ] );
  }
}
