"use strict";
{
  const T = new Proxy({},{
    get( _, prop ) {
      return new FutureType( prop );
    }
  });
  const typeNameMatcher = /\[object ([^\]]+)]/;
  module.exports = { T, FutureType, execute, typeOf };

  async function execute( data, v ) {
    return await new Promise( async (res,rej) => {
      const stack = Array.from(v.$$stack);
      let result = data;
      let last_result = data;
      for( const next of stack ) {
        if ( next.call ) {
          if ( next.func_name ) {
            result = result[next.func_name];
          }
          result = result.call( last_result, ...next.args );
        } else {
          result = result[next];
        }
        if ( Array.isArray( result ) ) {
          result = await Promise.all( result );
        } else if ( typeOf( result ) == "Promise" ) {
          result = await result; 
        }
        last_result = result;
      }
      res( result );
    });
  }

  function Handler( target ) {
    if ( ! new.target ) return new Handler( target );
    const p = new Proxy( target, this );
    Object.assign( this, {
      get( _, prop ) {
        if ( prop == Symbol.toStringTag || prop == Symbol.toPrimitive ) {
          return () => `proxy`;
        }
        if ( prop == Symbol.for('FutureType') ) {
          return 'FutureType';
        }
        if ( prop == "$$stack" ) {
          return target.stack;
        }
        target.stack.push(prop); 
        return p;
      },
      apply( _, that, args ) {
        const func_name = target.stack.pop();
        target.stack.push( { func_name, call: true, args } );
        return p;
      }
    }); 
  }

  function FutureType( prop ) {
    if ( ! new.target ) return new FutureType( prop );
    const callable = function callable() {};
    callable.prototype = this.__proto__; // why not
    callable.root = prop;
    callable.stack = [prop];
    const callable_proxy = new Proxy( callable, new Handler( callable ) );
    return callable_proxy;
  }

  function typeOf( thing ) {
    if ( !! thing && thing[Symbol.for('FutureType')] ) return "FutureType";
    const cname = !! thing && thing.constructor ? thing.constructor.name : null;
    const tname = typeNameMatcher.exec( Object.prototype.toString.call( thing ) )[1];
    if ( tname !== cname && !! cname ) return cname;
    return tname;
  }

}
