"use strict";
{
  const human = new Proxy( {}, {
    get( target, prop, receiver ) {
      // can accept
      // <event_name>( 'selector' )
      // <event_name>( [target_el1, target_el2 ])
      // <event_name>( target_el )
      // <event_name>( ) // defaults to self ( window )
      return async function( eventTarget = self, promise_list, index ) {
        if ( typeof eventTarget == "string" ) {
          eventTarget = Array.from( document.querySelectorAll(eventTarget) );
        } else if ( Array.isArray( eventTarget ) ) {
          eventTarget = Array.from( eventTarget );
        } else if ( !eventTarget ) {
          eventTarget = [self];
        } else {
          eventTarget = [eventTarget];
        }
        if ( Array.isArray( promise_list ) ) {
          // we only need to call this once and events will keep resolving
          // promises in our list
          const first_promise = new Promise( first_resolve => {
            eventTarget.forEach( 
              t => t.addEventListener( prop, event => {
                const last_promise = promise_list[index];
                if ( last_promise && last_promise.resolve ) {
                  last_promise.resolve(event);
                } else {
                  first_resolve(event);
                }
                let next_resolve;
                const next_promise = new Promise( resolve => next_resolve = resolve );
                next_promise.resolve = next_resolve;
                promise_list[index] = next_promise;
              })
            ); // forEach
          });     
          return first_promise;
        } else {
          return new Promise( (resolve, reject) => {
            eventTarget.forEach( t => t.addEventListener( prop, event => resolve(event), { once: true }) );
          });
        }
      }
    }
  });

  module.exports = human;
}
