"use strict";
{
  const human = require('./human.js');

  let app_running = false;

  const mon = {
    app_running, monitor 
  };

  module.exports = mon;

  function clean_pairs( pairs ) {
    return pairs.map( p => {
      if ( Array.isArray( p ) ) {
        if ( p.length <= 2 ) {
          return p;
        } else {
          return [p.shift(), p];
        }
      } else {
        return [p];
      }
    });
  }
  
  function make_first_promises( ps ) {
    ps.forEach( ([name,target],index) => {
      ps[index] = human[name](target,ps,index);
    });
    return ps;
  }

  async function monitor( classifier, ...event_target_pairs ) {
    app_running = true;
    event_target_pairs = clean_pairs( event_target_pairs );
    // notes
      // we add promises to the list 1 time
      // for each event
      // and then, when any promise resolves
      // that promise is removed from the list
      // and replaced, in the same position, with a new promise
      // that resolves on the same event, the next time it occurs
      // this is why we only have to add promises once to the list
      // and then can just call Promise.race( <list> ) repeatedly
      // since the list is updated ( by exactly one position ) each time
      // this updating of the list happens inside the actual event handler
      // function for the events we are monitoring
      // and this behaviour is defined in human.js
    const event_promises = make_first_promises( event_target_pairs );
    while( app_running ) {
      const event = await Promise.race( event_promises );
      classifier( event );
    }
  }
}

