"use strict";
{
  const {def, T, I} = require('./hil.js');
  const se = require('./semantic_events.js');
  const { dom1, domn, adomn } = require('./dom.js');

  module.exports = { run_test, test_templating, test_se };

  function test_templating() {
    const empty = def``;

    const banner = def`banner ${{file: 'banner.html' , stylesheet: 'xyz.css'}}`;

    const price = def` price_widget ${0}
              <div class=price>
                Price <span class=currency_symbol>${ d => d.currency == 'USD' ? '<em>USD</em>' : d.currency }</span> 
                <input class=value type=number value=${T.value}>
              </div>
    `;

    const item_line = def` item_line_widget ${0}
          <tr class=item_line>
            <td class=item_name>${T.item_name}${d => empty(d)}</td>
            <td class=item_price>
              ${I.price_widget(T.item_price)}
            </td>
            <td class=item_desc>${T.item_desc}</td>
          </tr>
    `;

    const items = def` items_widget ${{requests_iframe: true, iframe_name:'cris', width: '100%', height: 450 }}
      Banner ? 
      <hr>
      ${ I.banner(T) }
      <hr>
      <script src=app.js></script>
      <table>
        <tbody>
          ${T.items.map( i => I.item_line_widget(i) ).join('') }
        </tbody>
      </table>
      <script>
        if ( ! self.testing_se ) {
          test.test_se();
        }
      </script>
    `;

    items({ 
      link: 'https://2yago.com',
      linktext: 'The Rule of The World',
      paragraph: "Soon you shall see",
      items: [ 
        {
          item_name: 'Apple',
          item_price: {
            currency: 'USD',
            value: 12.00
          },
          item_desc: "Delicious & tasty"
        },
        {
          item_name: 'Pear',
          item_price: {
            currency: 'THB',
            value: 880 
          },
          item_desc: "Sweet and Juicy"
        },
        {
          item_name: 'Mangosteen',
          item_price: {
            currency: 'CNY',
            value: 18 
          },
          item_desc: "Exotic and Rich"
        }
      ]
    }).then( x => (console.log(x), setTimeout( () => document.body.appendChild( hil.renderHTML( x ) ), 0 )) );
  }
  
  async function test_se() {
    self.testing_se = true;
    se.start_monitoring( 
      'click', 'textInput', [ 'focus', 'input' ], 
      [ 'select', await adomn('input', 500 ) ], 'focus',
      'copy', 'cut', 'paste', 
      'beforecut', 'beforecopy', 'beforepaste',
      'input',
      [ 'mousemove', await adomn('h1', 500 )],
      'scroll',
      'change' );
  }

  async function run_test() {
    test_templating();
    await test_se();
  }
}
